@@ -2,7 +2,6 @@
 import urllib.parse
 import asyncio
 import json
-import logging
 import importlib
 import stdlibs
 import sys
@@ -17,22 +16,23 @@ from dataclasses import asdict
 from typing import Any, Dict, Coroutine, Optional
 
 from .prompts import LLM_SYSTEM_PROMPT
-from .core.database import DatabaseManager
+from .core import get_database_manager
 from .core.jobs import JobManager, JobStatus
 from .core.watcher import CodeWatcher
 from .tools.graph_builder import GraphBuilder
 from .tools.code_finder import CodeFinder
-from .tools.import_extractor import ImportExtractor
-from .utils.debug_log import debug_log
+from .tools.package_resolver import get_local_package_path
+from .utils.debug_log import debug_log, info_logger, error_logger, warning_logger, debug_logger
 
-logger = logging.getLogger(__name__)
+DEFAULT_EDIT_DISTANCE = 2
+DEFAULT_FUZZY_SEARCH = False
 
 class MCPServer:
     """
     The main MCP Server class.
     
     This class orchestrates all the major components of the application, including:
-    - Database connection management (`DatabaseManager`)
+    - Database connection management (`DatabaseManager` or `FalkorDBManager`)
     - Background job tracking (`JobManager`)
     - File system watching for live updates (`CodeWatcher`)
     - Tool handlers for graph building, code searching, etc.
@@ -48,9 +48,9 @@ class MCPServer:
                   running loop or creates a new one.
         """
         try:
-            # Initialize the database manager and establish a connection early
-            # to fail fast if credentials are wrong.
-            self.db_manager = DatabaseManager()
+            # Initialize the database manager (Neo4j or FalkorDB Lite based on env var)
+            # to fail fast if credentials/configuration are wrong.
+            self.db_manager = get_database_manager()
             self.db_manager.get_driver() 
         except ValueError as e:
             raise ValueError(f"Database configuration error: {e}")
@@ -70,7 +70,6 @@ class MCPServer:
         # Initialize all the tool handlers, passing them the necessary managers and the event loop.
         self.graph_builder = GraphBuilder(self.db_manager, self.job_manager, loop)
         self.code_finder = CodeFinder(self.db_manager)
-        self.import_extractor = ImportExtractor()
         self.code_watcher = CodeWatcher(self.graph_builder, self.job_manager)
         
         # Define the tool manifest that will be exposed to the AI assistant.
@@ -109,15 +108,16 @@ class MCPServer:
                 "description": "List all background jobs and their current status.",
                 "inputSchema": {"type": "object", "properties": {}}
             },
-            "find_code": {
+           "find_code": {
                 "name": "find_code",
                 "description": "Find relevant code snippets related to a keyword (e.g., function name, class name, or content).",
                 "inputSchema": {
                     "type": "object",
-                    "properties": { "query": {"type": "string", "description": "Keyword or phrase to search for"} },
+                    "properties": { "query": {"type": "string", "description": "Keyword or phrase to search for"}, "fuzzy_search": {"type": "boolean", "description": "Whether to use fuzzy search", "default": False}, "edit_distance": {"type": "number", "description": "Edit distance for fuzzy search (between 0-2)", "default": 2}}, 
                     "required": ["query"]
                 }
             },
+
             "analyze_code_relationships": {
                 "name": "analyze_code_relationships",
                 "description": "Analyze code relationships like 'who calls this function' or 'class hierarchy'. Supported query types include: find_callers, find_callees, find_all_callers, find_all_callees, find_importers, who_modifies, class_hierarchy, overrides, dead_code, call_chain, module_deps, variable_scope, find_complexity, find_functions_by_argument, find_functions_by_decorator.",
@@ -151,27 +151,15 @@ class MCPServer:
             },
             "add_package_to_graph": {
                 "name": "add_package_to_graph",
-                "description": "Add a Python package to Neo4j graph by discovering its location. Returns immediately with job ID.",
+                "description": "Add a package to the graph by discovering its location. Supports multiple languages. Returns immediately with a job ID.",
                 "inputSchema": {
                     "type": "object",
                     "properties": {
-                        "package_name": {"type": "string", "description": "Name of the Python package to add (e.g., 'requests')"},
-                        "is_dependency": {"type": "boolean", "description": "Mark as a dependency", "default": True}
+                        "package_name": {"type": "string", "description": "Name of the package to add (e.g., 'requests', 'express', 'moment', 'lodash')."},
+                        "language": {"type": "string", "description": "The programming language of the package.", "enum": ["python", "javascript", "typescript", "java", "c", "go", "ruby", "php","cpp"]},
+                        "is_dependency": {"type": "boolean", "description": "Mark as a dependency.", "default": True}
                     },
-                    "required": ["package_name"]
-                }
-            },
-            "list_imports": {
-                "name": "list_imports",
-                "description": "Extract all package imports from code files in a directory or file",
-                "inputSchema": {
-                    "type": "object",
-                    "properties": {
-                        "path": {"type": "string", "description": "Path to file or directory to analyze"},
-                        "language": {"type": "string", "description": "Programming language (python, javascript, etc.)", "default": "python"},
-                        "recursive": {"type": "boolean", "description": "Whether to analyze subdirectories recursively", "default": True}
-                    },
-                    "required": ["path"]
+                    "required": ["package_name", "language"]
                 }
             },
             "find_dead_code": {
@@ -258,64 +246,7 @@ class MCPServer:
         """Returns the current connection status of the Neo4j database."""
         return {"connected": self.db_manager.is_connected()}
         
-    def get_local_package_path(self, package_name: str) -> Optional[str]:
-        """
-        Finds the local installation path of a Python package.
 
-        This method uses `importlib` to locate a package and determines its root
-        directory, handling both regular packages (directories with __init__.py)
-        and single-file modules.
-
-        Args:
-            package_name: The name of the package to locate (e.g., "requests").
-
-        Returns:
-            The absolute path to the package's directory as a string, or None if not found.
-        """
-        try:
-            debug_log(f"Getting local path for package: {package_name}")
-            
-            module = importlib.import_module(package_name)
-            
-            if hasattr(module, '__file__') and module.__file__:
-                module_file = Path(module.__file__)
-                debug_log(f"Module file: {module_file}")
-
-                if module_file.name == '__init__.py':
-                    # For a package, the path is the parent directory of __init__.py
-                    package_path = str(module_file.parent)
-                elif package_name in stdlibs.module_names:
-                    # For a standard library single file module, the path is the file itself
-                    package_path = str(module_file)
-                else:
-                    # For other single-file modules, assume the parent directory is the container
-                    package_path = str(module_file.parent)
-
-                debug_log(f"Determined package path: {package_path}")
-                return package_path
-
-            elif hasattr(module, '__path__'):
-                # This handles namespace packages which may not have an __init__.py
-                if isinstance(module.__path__, list) and module.__path__:
-                    package_path = str(Path(module.__path__[0]))
-                    debug_log(f"Package path from __path__: {package_path}")
-                    return package_path
-                else:
-                    # Fallback for other __path__ formats
-                    package_path = str(Path(str(module.__path__)))
-                    debug_log(f"Package path from __path__ (str): {package_path}")
-                    return package_path
-            
-            debug_log(f"Could not determine path for {package_name}")
-            return None
-            
-        except ImportError as e:
-            debug_log(f"Could not import {package_name}: {e}")
-            return None
-        except Exception as e:
-            debug_log(f"Error getting local path for {package_name}: {e}")
-            return None
-        
     def execute_cypher_query_tool(self, **args) -> Dict[str, Any]:
         """
         Tool implementation for executing a read-only Cypher query.
@@ -443,33 +374,137 @@ class MCPServer:
         repo_path = args.get("repo_path")
         try:
             debug_log(f"Deleting repository: {repo_path}")
-            self.graph_builder.delete_repository_from_graph(repo_path)
-            return {
-                "success": True,
-                "message": f"Repository '{repo_path}' deleted successfully."
-            }
+            if self.graph_builder.delete_repository_from_graph(repo_path):
+                return {
+                    "success": True,
+                    "message": f"Repository '{repo_path}' deleted successfully."
+                }
+            else:
+                 return {
+                    "success": False,
+                    "message": f"Repository '{repo_path}' not found in the graph."
+                }
         except Exception as e:
             debug_log(f"Error deleting repository: {str(e)}")
             return {"error": f"Failed to delete repository: {str(e)}"}
 
     def visualize_graph_query_tool(self, **args) -> Dict[str, Any]:
-        """Tool to generate a Neo4j browser visualization URL for a Cypher query."""
+        """Tool to generate a visualization URL (Neo4j URL or FalkorDB HTML file)."""
         cypher_query = args.get("cypher_query")
         if not cypher_query:
             return {"error": "Cypher query cannot be empty."}
 
-        try:
-            encoded_query = urllib.parse.quote(cypher_query)
-            visualization_url = f"http://localhost:7474/browser/?cmd=edit&arg={encoded_query}"
-            
-            return {
-                "success": True,
-                "visualization_url": visualization_url,
-                "message": "Open the URL in your browser to visualize the graph query. The query will be pre-filled for editing."
-            }
-        except Exception as e:
-            debug_log(f"Error generating visualization URL: {str(e)}")
-            return {"error": f"Failed to generate visualization URL: {str(e)}"}
+        # Check DB Type: FalkorDBManager vs DatabaseManager
+        is_falkor = "FalkorDB" in self.db_manager.__class__.__name__
+
+        if is_falkor:
+            try:
+                data_nodes = []
+                data_edges = []
+                seen_nodes = set()
+
+                with self.db_manager.get_driver().session() as session:
+                    result = session.run(cypher_query)
+                    for record in result:
+                        # Iterate all values in the record to find Nodes and Relationships
+                        # record is a FalkorDBRecord (dict-like), values() works
+                        for val in record.values():
+                            # Process Node
+                            if hasattr(val, 'labels') and hasattr(val, 'id'):
+                                nid = val.id
+                                if nid not in seen_nodes:
+                                    seen_nodes.add(nid)
+                                    lbl = list(val.labels)[0] if val.labels else "Node"
+                                    props = getattr(val, 'properties', {}) or {}
+                                    name = props.get('name', str(nid))
+                                    
+                                    color = "#97c2fc"
+                                    if "Repository" in val.labels: color = "#ffb3ba"
+                                    elif "File" in val.labels: color = "#baffc9"
+                                    elif "Class" in val.labels: color = "#bae1ff"
+                                    elif "Function" in val.labels: color = "#ffffba"
+                                    
+                                    data_nodes.append({
+                                        "id": nid, "label": name, "group": lbl, 
+                                        "title": str(props), "color": color
+                                    })
+                            
+                            # Process Relationship
+                            # FalkorDB client relationship objects have src_node/dest_node (ids) and relation (type)
+                            # We check for both standard names just in case
+                            src = getattr(val, 'src_node', None)
+                            if src is None: src = getattr(val, 'start_node', None)
+                            
+                            dst = getattr(val, 'dest_node', None)
+                            if dst is None: dst = getattr(val, 'end_node', None)
+
+                            if src is not None and dst is not None:
+                                lbl = getattr(val, 'relation', None) or getattr(val, 'type', 'REL')
+                                data_edges.append({
+                                    "from": src,
+                                    "to": dst,
+                                    "label": lbl,
+                                    "arrows": "to"
+                                })
+
+                # Generate HTML
+                html_content = f"""
+<!DOCTYPE html>
+<html>
+<head>
+  <title>CodeGraphContext Visualization</title>
+  <script type="text/javascript" src="https://unpkg.com/vis-network/standalone/umd/vis-network.min.js"></script>
+  <style type="text/css">
+    #mynetwork {{ width: 100%; height: 100vh; border: 1px solid lightgray; }}
+  </style>
+</head>
+<body>
+  <div id="mynetwork"></div>
+  <script type="text/javascript">
+    var nodes = new vis.DataSet({json.dumps(data_nodes)});
+    var edges = new vis.DataSet({json.dumps(data_edges)});
+    var container = document.getElementById('mynetwork');
+    var data = {{ nodes: nodes, edges: edges }};
+    var options = {{
+        nodes: {{ shape: 'dot', size: 16 }},
+        physics: {{ stabilization: false }},
+        layout: {{ improvedLayout: false }}
+    }};
+    var network = new vis.Network(container, data, options);
+  </script>
+</body>
+</html>
+"""
+                # filename = f"codegraph_viz_{int(datetime.now().timestamp())}.html"
+                filename = f"codegraph_viz.html"
+                out_path = Path(os.getcwd()) / filename
+                with open(out_path, "w") as f:
+                    f.write(html_content)
+                
+                return {
+                    "success": True,
+                    "visualization_url": f"file://{out_path}",
+                    "message": f"Visualization generated at {out_path}. Open this file in your browser."
+                }
+
+            except Exception as e:
+                debug_log(f"Error generating FalkorDB visualization: {str(e)}")
+                return {"error": f"Failed to generate visualization: {str(e)}"}
+
+        else:
+            # Neo4j fallback
+            try:
+                encoded_query = urllib.parse.quote(cypher_query)
+                visualization_url = f"http://localhost:7474/browser/?cmd=edit&arg={encoded_query}"
+                
+                return {
+                    "success": True,
+                    "visualization_url": visualization_url,
+                    "message": "Open the URL in your browser to visualize the graph query. The query will be pre-filled for editing."
+                }
+            except Exception as e:
+                debug_log(f"Error generating visualization URL: {str(e)}")
+                return {"error": f"Failed to generate visualization URL: {str(e)}"}
 
     def list_watched_paths_tool(self, **args) -> Dict[str, Any]:
         """Tool to list all currently watched directory paths."""
@@ -540,55 +575,8 @@ class MCPServer:
                 }
             
         except Exception as e:
-            logger.error(f"Failed to start watching directory {path}: {e}")
+            error_logger(f"Failed to start watching directory {path}: {e}")
             return {"error": f"Failed to start watching directory: {str(e)}"}        
-    def list_imports_tool(self, **args) -> Dict[str, Any]:
-        """Tool to list all imports from code files"""        
-        path = args.get("path")
-        language = args.get("language", "python")
-        recursive = args.get("recursive", True)
-        all_imports = set()
-        file_extensions = {
-            'python': ['.py'], 'javascript': ['.js', '.jsx', '.mjs'],
-            'typescript': ['.ts', '.tsx'], 'java': ['.java'],
-        }
-        
-        extensions = file_extensions.get(language, ['.py'])
-        extract_func = {
-            'python': self.import_extractor.extract_python_imports,
-            'javascript': self.import_extractor.extract_javascript_imports,
-            'typescript': self.import_extractor.extract_javascript_imports,
-            'java': self.import_extractor.extract_java_imports,
-        }.get(language, self.import_extractor.extract_python_imports)
-        
-        try:
-            path_obj = Path(path)
-            
-            if path_obj.is_file():
-                if any(str(path_obj).endswith(ext) for ext in extensions):
-                    all_imports.update(extract_func(str(path_obj)))
-            elif path_obj.is_dir():
-                pattern = "**/*" if recursive else "*"
-                for ext in extensions:
-                    for file_path in path_obj.glob(f"{pattern}{ext}"):
-                        if file_path.is_file():
-                            all_imports.update(extract_func(str(file_path)))
-            else:
-                return {"error": f"Path {path} does not exist"}
-            
-            # Removed standard library filtering as per user request.
-            # if language == 'python':
-            #     # Get the list of stdlib modules for the current Python version
-            #     stdlib_modules = set(stdlibs.module_names)
-            #     all_imports = all_imports - stdlib_modules
-            
-            return {
-                "imports": sorted(list(all_imports)), "language": language,
-                "path": path, "count": len(all_imports)
-            }
-        
-        except Exception as e:
-            return {"error": f"Failed to analyze imports: {str(e)}"}
     
     def add_code_to_graph_tool(self, **args) -> Dict[str, Any]:
         """
@@ -646,10 +634,14 @@ class MCPServer:
             return {"error": f"Failed to start background processing: {str(e)}"}
     
     def add_package_to_graph_tool(self, **args) -> Dict[str, Any]:
-        """Tool to add a Python package to Neo4j graph by auto-discovering its location"""
+        """Tool to add a package to the graph by auto-discovering its location"""
         package_name = args.get("package_name")
+        language = args.get("language")
         is_dependency = args.get("is_dependency", True)
-        
+
+        if not language:
+            return {"error": "The 'language' parameter is required."}
+
         try:
             # Check if the package is already indexed
             indexed_repos = self.list_indexed_repositories_tool().get("repositories", [])
@@ -660,10 +652,10 @@ class MCPServer:
                         "message": f"Package '{package_name}' is already indexed."
                     }
 
-            package_path = self.get_local_package_path(package_name)
+            package_path = get_local_package_path(package_name, language)
             
             if not package_path:
-                return {"error": f"Could not find package '{package_name}'. Make sure it's installed."}
+                return {"error": f"Could not find package '{package_name}' for language '{language}'. Make sure it's installed."}
             
             if not os.path.exists(package_path):
                 return {"error": f"Package path '{package_path}' does not exist"}
@@ -799,15 +791,24 @@ class MCPServer:
         except Exception as e:
             debug_log(f"Error analyzing relationships: {str(e)}")
             return {"error": f"Failed to analyze relationships: {str(e)}"}
+        
+    @staticmethod
+    def _normalize(text: str) -> str:
+        return text.lower().replace("_", " ").strip()
 
     def find_code_tool(self, **args) -> Dict[str, Any]:
         """Tool to find relevant code snippets"""
         query = args.get("query")
-        
-        try:
-            debug_log(f"Finding code for query: {query}")
-            results = self.code_finder.find_related_code(query)
+        fuzzy_search = args.get("fuzzy_search", DEFAULT_FUZZY_SEARCH)
+        edit_distance = args.get("edit_distance", DEFAULT_EDIT_DISTANCE)
+
+        if fuzzy_search:
+            query = self._normalize(query)
             
+        try:
+            debug_log(f"Finding code for query: {query} with fuzzy_search={fuzzy_search}, edit_distance={edit_distance}")
+            results = self.code_finder.find_related_code(query, fuzzy_search, edit_distance)
+
             return {"success": True, "query": query, "results": results}
         
         except Exception as e:
@@ -827,7 +828,6 @@ class MCPServer:
             A dictionary containing the result of the tool execution.
         """
         tool_map: Dict[str, Coroutine] = {
-            "list_imports": self.list_imports_tool,
             "add_package_to_graph": self.add_package_to_graph_tool,
             "find_dead_code": self.find_dead_code_tool,
             "find_code": self.find_code_tool,
@@ -861,7 +861,7 @@ class MCPServer:
         requests, and routes them to the appropriate handlers (e.g., initialize,
         tools/list, tools/call). The response is then printed to stdout.
         """
-        logger.info("MCP Server is running. Waiting for requests...")
+        debug_logger("MCP Server is running. Waiting for requests...")
         self.code_watcher.start()
         
         loop = asyncio.get_event_loop()
@@ -870,7 +870,7 @@ class MCPServer:
                 # Read a request from the standard input.
                 line = await loop.run_in_executor(None, sys.stdin.readline)
                 if not line:
-                    logger.info("Client disconnected (EOF received). Shutting down.")
+                    debug_logger("Client disconnected (EOF received). Shutting down.")
                     break
                 
                 request = json.loads(line.strip())
@@ -930,7 +930,7 @@ class MCPServer:
                     print(json.dumps(response), flush=True)
 
             except Exception as e:
-                logger.error(f"Error processing request: {e}\n{traceback.format_exc()}")
+                error_logger(f"Error processing request: {e}\n{traceback.format_exc()}")
                 request_id = "unknown"
                 if 'request' in locals() and isinstance(request, dict):
                     request_id = request.get('id', "unknown")
@@ -943,6 +943,6 @@ class MCPServer:
 
     def shutdown(self):
         """Gracefully shuts down the server and its components."""
-        logger.info("Shutting down server...")
+        debug_logger("Shutting down server...")
         self.code_watcher.stop()
         self.db_manager.close_driver()
